@@ -12,7 +12,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'AST_H': 'Include/Python-ast.h',
  'AST_H_DIR': 'Include',
  'BASECFLAGS': '-Wno-unused-result -Wsign-compare',
- 'BASECPPFLAGS': '',
+ 'BASECPPFLAGS': '-IObjects -IInclude -IPython',
  'BASEMODLIBS': '',
  'BINDIR': '/usr/bin',
  'BINLIBDEST': '/usr/lib/python3.5',
@@ -26,14 +26,17 @@ build_time_vars = {'ABIFLAGS': 'm',
  'CC': 'x86_64-linux-gnu-gcc -pthread',
  'CCSHARED': '-fPIC',
  'CFLAGS': '-Wno-unused-result -Wsign-compare -DNDEBUG -g -fwrapv -O2 -Wall '
-           '-Wstrict-prototypes -g -fstack-protector-strong -Wformat '
-           '-Werror=format-security  -g -flto -fuse-linker-plugin '
-           '-ffat-lto-objects',
+           '-Wstrict-prototypes -g '
+           '-fdebug-prefix-map=/build/python3.5-6tVwKN/python3.5-3.5.2=. '
+           '-fstack-protector-strong -Wformat -Werror=format-security  -g '
+           '-flto -fuse-linker-plugin -ffat-lto-objects',
  'CFLAGSFORSHARED': '-fPIC',
  'CFLAGS_NODIST': '',
  'CONFIGFILES': 'configure configure.ac acconfig.h pyconfig.h.in '
                 'Makefile.pre.in',
- 'CONFIGURE_CFLAGS': '-g -fstack-protector-strong -Wformat '
+ 'CONFIGURE_CFLAGS': '-g '
+                     '-fdebug-prefix-map=/build/python3.5-6tVwKN/python3.5-3.5.2=. '
+                     '-fstack-protector-strong -Wformat '
                      '-Werror=format-security',
  'CONFIGURE_CFLAGS_NODIST': '-Werror=declaration-after-statement',
  'CONFIGURE_CPPFLAGS': '-Wdate-time -D_FORTIFY_SOURCE=2',
@@ -43,18 +46,20 @@ build_time_vars = {'ABIFLAGS': 'm',
                 "'--with-dbmliborder=bdb:gdbm' '--with-computed-gotos' "
                 "'--without-ensurepip' '--with-system-expat' "
                 "'--with-system-libmpdec' '--with-system-ffi' '--with-fpectl' "
-                "'CC=x86_64-linux-gnu-gcc' 'CFLAGS=-g -fstack-protector-strong "
-                "-Wformat -Werror=format-security ' "
+                "'CC=x86_64-linux-gnu-gcc' 'CFLAGS=-g "
+                '-fdebug-prefix-map=/build/python3.5-6tVwKN/python3.5-3.5.2=. '
+                "-fstack-protector-strong -Wformat -Werror=format-security ' "
                 "'LDFLAGS=-Wl,-Bsymbolic-functions -Wl,-z,relro' "
                 "'CPPFLAGS=-Wdate-time -D_FORTIFY_SOURCE=2'",
  'CONFINCLUDEDIR': '/usr/include',
  'CONFINCLUDEPY': '/usr/include/python3.5m',
  'COREPYTHONPATH': ':plat-x86_64-linux-gnu',
- 'COVERAGE_INFO': '/build/python3.5-9imW1d/python3.5-3.5.2/build-shared/coverage.info',
- 'COVERAGE_REPORT': '/build/python3.5-9imW1d/python3.5-3.5.2/build-shared/lcov-report',
+ 'COVERAGE_INFO': '/build/python3.5-6tVwKN/python3.5-3.5.2/build-shared/coverage.info',
+ 'COVERAGE_REPORT': '/build/python3.5-6tVwKN/python3.5-3.5.2/build-shared/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov '
                             'report"',
- 'CPPFLAGS': '-I. -IInclude -I../Include -Wdate-time -D_FORTIFY_SOURCE=2',
+ 'CPPFLAGS': '-IObjects -IInclude -IPython -I. -I../Include -Wdate-time '
+             '-D_FORTIFY_SOURCE=2',
  'CXX': 'x86_64-linux-gnu-g++ -pthread',
  'DESTDIRS': '/usr /usr/lib /usr/lib/python3.5 /usr/lib/python3.5/lib-dynload',
  'DESTLIB': '/usr/lib/python3.5',
@@ -303,7 +308,6 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_PTY_H': 1,
  'HAVE_PUTENV': 1,
  'HAVE_PWRITE': 1,
- 'HAVE_RAND_EGD': 1,
  'HAVE_READLINK': 1,
  'HAVE_READLINKAT': 1,
  'HAVE_READV': 1,
@@ -557,7 +561,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'OPCODETARGETS_H': '\\',
  'OPCODE_H': '../Include/opcode.h',
  'OPCODE_H_DIR': '../Include',
- 'OPCODE_H_GEN': 'python3.5  ../Tools/scripts/generate_opcode_h.py '
+ 'OPCODE_H_GEN': 'python3.5 ../Tools/scripts/generate_opcode_h.py '
                  '../Lib/opcode.py ../Include/opcode.h',
  'OPCODE_H_SCRIPT': '../Tools/scripts/generate_opcode_h.py',
  'OPT': '-DNDEBUG -g -fwrapv -O2 -Wall -Wstrict-prototypes',
@@ -596,21 +600,25 @@ build_time_vars = {'ABIFLAGS': 'm',
  'PYTHONFRAMEWORKPREFIX': '',
  'PYTHONPATH': ':plat-x86_64-linux-gnu',
  'PYTHON_FOR_BUILD': './python -E',
+ 'PYTHON_FOR_GEN': 'python3.5',
  'PYTHON_HEADERS': '\\',
  'PYTHON_OBJS': '\\',
  'PY_CFLAGS': '-Wno-unused-result -Wsign-compare -DNDEBUG -g -fwrapv -O2 -Wall '
-              '-Wstrict-prototypes -g -fstack-protector-strong -Wformat '
-              '-Werror=format-security  -g -flto -fuse-linker-plugin '
-              '-ffat-lto-objects',
+              '-Wstrict-prototypes -g '
+              '-fdebug-prefix-map=/build/python3.5-6tVwKN/python3.5-3.5.2=. '
+              '-fstack-protector-strong -Wformat -Werror=format-security  -g '
+              '-flto -fuse-linker-plugin -ffat-lto-objects',
  'PY_CFLAGS_NODIST': '-Werror=declaration-after-statement',
  'PY_CORE_CFLAGS': '-Wno-unused-result -Wsign-compare -DNDEBUG -g -fwrapv -O2 '
-                   '-Wall -Wstrict-prototypes -g -fstack-protector-strong '
-                   '-Wformat -Werror=format-security  -g -flto '
-                   '-fuse-linker-plugin -ffat-lto-objects '
-                   '-Werror=declaration-after-statement -I. -IInclude '
-                   '-I../Include -Wdate-time -D_FORTIFY_SOURCE=2 -fPIC '
-                   '-DPy_BUILD_CORE',
- 'PY_CPPFLAGS': '-I. -IInclude -I../Include -Wdate-time -D_FORTIFY_SOURCE=2',
+                   '-Wall -Wstrict-prototypes -g '
+                   '-fdebug-prefix-map=/build/python3.5-6tVwKN/python3.5-3.5.2=. '
+                   '-fstack-protector-strong -Wformat -Werror=format-security  '
+                   ' '
+                   '-Werror=declaration-after-statement -IObjects -IInclude '
+                   '-IPython -I. -I../Include -Wdate-time -D_FORTIFY_SOURCE=2 '
+                   '-fPIC -DPy_BUILD_CORE',
+ 'PY_CPPFLAGS': '-IObjects -IInclude -IPython -I. -I../Include -Wdate-time '
+                '-D_FORTIFY_SOURCE=2',
  'PY_FORMAT_LONG_LONG': '"ll"',
  'PY_FORMAT_SIZE_T': '"z"',
  'PY_LDFLAGS': '-Wl,-Bsymbolic-functions -Wl,-z,relro',
@@ -622,7 +630,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/build/python3.5-9imW1d/python3.5-3.5.2/build-shared',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/build/python3.5-6tVwKN/python3.5-3.5.2/build-shared',
  'SCRIPTDIR': '/usr/lib',
  'SETPGRP_HAVE_ARG': 0,
  'SGI_ABI': '',
@@ -664,10 +672,10 @@ build_time_vars = {'ABIFLAGS': 'm',
  'TCLTK_LIBS': '',
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/python3.5-9imW1d/python3.5-3.5.2/build-shared '
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/python3.5-6tVwKN/python3.5-3.5.2/build-shared '
                './python',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/python3.5-9imW1d/python3.5-3.5.2/build-shared '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/python3.5-6tVwKN/python3.5-3.5.2/build-shared '
                './python ../Tools/scripts/run_tests.py',
  'TESTTIMEOUT': 3600,
  'THREADOBJ': 'Python/thread.o',
@@ -693,9 +701,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/build/python3.5-9imW1d/python3.5-3.5.2/build-shared',
- 'abs_srcdir': '/build/python3.5-9imW1d/python3.5-3.5.2/build-shared/..',
- 'cross_compiling': 'no',
+ 'abs_builddir': '/build/python3.5-6tVwKN/python3.5-3.5.2/build-shared',
+ 'abs_srcdir': '/build/python3.5-6tVwKN/python3.5-3.5.2/build-shared/..',
  'datarootdir': '/usr/share',
  'exec_prefix': '/usr',
  'prefix': '/usr',
